@@ -6,8 +6,10 @@ import os
 import sys
 import types
 
+import six
+
 import cherrypy
-from cherrypy._cpcompat import itervalues, ntob, ntou
+from cherrypy._cpcompat import ntob, ntou
 from cherrypy import _cptools, tools
 from cherrypy.lib import httputil, static
 
@@ -55,7 +57,7 @@ class CoreRequestHandlingTest(helper.CPWebCase):
             """
             def __init__(cls, name, bases, dct):
                 type.__init__(cls, name, bases, dct)
-                for value in itervalues(dct):
+                for value in six.itervalues(dct):
                     if isinstance(value, types.FunctionType):
                         value.exposed = True
                 setattr(root, name.lower(), cls())
@@ -74,6 +76,9 @@ class CoreRequestHandlingTest(helper.CPWebCase):
                     relative = bool(relative)
                 return cherrypy.url(path_info, relative=relative)
 
+            def qs(self, qs):
+                return cherrypy.url(qs=qs)
+
         def log_status():
             Status.statuses.append(cherrypy.response.status)
         cherrypy.tools.log_status = cherrypy.Tool(
@@ -150,6 +155,10 @@ class CoreRequestHandlingTest(helper.CPWebCase):
             def url_with_quote(self):
                 raise cherrypy.HTTPRedirect("/some\"url/that'we/want")
 
+            def url_with_xss(self):
+                raise cherrypy.HTTPRedirect(
+                    "/some<script>alert(1);</script>url/that'we/want")
+
             def url_with_unicode(self):
                 raise cherrypy.HTTPRedirect(ntou('тест', 'utf-8'))
 
@@ -316,8 +325,10 @@ class CoreRequestHandlingTest(helper.CPWebCase):
         # Make sure GET params are preserved.
         self.getPage('/redirect?id=3')
         self.assertStatus(301)
-        self.assertMatchesBody('<a href=([\'"])%s/redirect/[?]id=3\\1>'
-                          '%s/redirect/[?]id=3</a>' % (self.base(), self.base()))
+        self.assertMatchesBody(
+            '<a href=([\'"])%s/redirect/[?]id=3\\1>'
+            '%s/redirect/[?]id=3</a>' % (self.base(), self.base())
+        )
 
         if self.prefix():
             # Corner case: the "trailing slash" redirect could be tricky if
@@ -325,16 +336,18 @@ class CoreRequestHandlingTest(helper.CPWebCase):
             self.getPage('')
             self.assertStatus(301)
             self.assertMatchesBody("<a href=(['\"])%s/\\1>%s/</a>" %
-                              (self.base(), self.base()))
+                                   (self.base(), self.base()))
 
         # Test that requests for NON-index methods WITH a trailing slash
         # get redirected to the same URI path WITHOUT a trailing slash.
         # Make sure GET params are preserved.
         self.getPage('/redirect/by_code/?code=307')
         self.assertStatus(301)
-        self.assertMatchesBody("<a href=(['\"])%s/redirect/by_code[?]code=307\\1>"
-                          '%s/redirect/by_code[?]code=307</a>'
-                          % (self.base(), self.base()))
+        self.assertMatchesBody(
+            "<a href=(['\"])%s/redirect/by_code[?]code=307\\1>"
+            '%s/redirect/by_code[?]code=307</a>'
+            % (self.base(), self.base())
+        )
 
         # If the trailing_slash tool is off, CP should just continue
         # as if the slashes were correct. But it needs some help
@@ -420,10 +433,14 @@ class CoreRequestHandlingTest(helper.CPWebCase):
         def assertValidXHTML():
             from xml.etree import ElementTree
             try:
-                ElementTree.fromstring('<html><body>%s</body></html>' % self.body)
-            except ElementTree.ParseError as e:
-                self._handlewebError('automatically generated redirect '
-                    'did not generate well-formed html')
+                ElementTree.fromstring(
+                    '<html><body>%s</body></html>' % self.body,
+                )
+            except ElementTree.ParseError:
+                self._handlewebError(
+                    'automatically generated redirect did not '
+                    'generate well-formed html',
+                )
 
         # check redirects to URLs generated valid HTML - we check this
         # by seeing if it appears as valid XHTML.
@@ -436,6 +453,14 @@ class CoreRequestHandlingTest(helper.CPWebCase):
         self.assertStatus(303)
         assertValidXHTML()
 
+    def test_redirect_with_xss(self):
+        """A redirect to a URL with HTML injected should result
+        in page contents escaped."""
+        self.getPage('/redirect/url_with_xss')
+        self.assertStatus(303)
+        assert b'<script>' not in self.body
+        assert b'&lt;script&gt;' in self.body
+
     def test_redirect_with_unicode(self):
         """
         A redirect to a URL with Unicode should return a Location
@@ -638,6 +663,8 @@ class CoreRequestHandlingTest(helper.CPWebCase):
         self.assertBody('%s/url/other/page1' % self.base())
         self.getPage('/url/?path_info=/other/./page1')
         self.assertBody('%s/other/page1' % self.base())
+        self.getPage('/url/?path_info=/other/././././page1')
+        self.assertBody('%s/other/page1' % self.base())
 
         # Double dots
         self.getPage('/url/leaf?path_info=../page1')
@@ -646,6 +673,20 @@ class CoreRequestHandlingTest(helper.CPWebCase):
         self.assertBody('%s/url/page1' % self.base())
         self.getPage('/url/leaf?path_info=/other/../page1')
         self.assertBody('%s/page1' % self.base())
+        self.getPage('/url/leaf?path_info=/other/../../../page1')
+        self.assertBody('%s/page1' % self.base())
+        self.getPage('/url/leaf?path_info=/other/../../../../../page1')
+        self.assertBody('%s/page1' % self.base())
+
+        # qs param is not normalized as a path
+        self.getPage('/url/qs?qs=/other')
+        self.assertBody('%s/url/qs?/other' % self.base())
+        self.getPage('/url/qs?qs=/other/../page1')
+        self.assertBody('%s/url/qs?/other/../page1' % self.base())
+        self.getPage('/url/qs?qs=../page1')
+        self.assertBody('%s/url/qs?../page1' % self.base())
+        self.getPage('/url/qs?qs=../../page1')
+        self.assertBody('%s/url/qs?../../page1' % self.base())
 
         # Output relative to current path or script_name
         self.getPage('/url/?path_info=page1&relative=True')
@@ -741,7 +782,7 @@ class ErrorTests(helper.CPWebCase):
             @cherrypy.expose
             def stat(self, path):
                 with cherrypy.HTTPError.handle(OSError, 404):
-                    st = os.stat(path)
+                    os.stat(path)
 
         root = Root()
 
@@ -756,7 +797,11 @@ class ErrorTests(helper.CPWebCase):
     def test_contextmanager(self):
         self.getPage('/stat/missing')
         self.assertStatus(404)
-        self.assertInBody('No such file or directory')
+        body_text = self.body.decode('utf-8')
+        assert (
+            'No such file or directory' in body_text or
+            'cannot find the file specified' in body_text
+        )
 
 
 class TestBinding:
